/*! \file affichage.c
    \brief Fonctions d'affichage

    Ensemble de fonctions permettant de crer et de modifier l'affichage du jeu

    \Author D.Bernaudeau
    \Author J. Vehent
 */

#include <ncurses.h>
#include <string.h>

/*!
   \var grille_adverse
   \brief Ensemble de cases pour la grille adverse
 */
WINDOW* grille_adverse[10][10];

/*!
   \var grille_locale
   \brief Ensemble de cases pour la grille du joueur
 */
WINDOW* grille_locale[10][10];

/*!
   \var bateau
   \brief Fenetre d'affiche centrale pour la liste des bateaux
 */
WINDOW* bateau;

/*!
   \var erreur
   \brief Fenetre d'affiche des messages d'erreur
 */
WINDOW* erreur;

//! Creer la fenetre pour accompagner la grille de jeu
/*!
\param[in] fenetre contour de la grille de jeu (A B C... et 1 2 3...)
\param[in,out] grille_jeu liste de fenetre representant les cases de la grille
\param[in] x coordonne du coin haut gauche de la fenetre de la grille (Colonne)
\param[in] y coordonne du coin haut gauche de la fenetre de la grille (ligne)
\param[in] choix 1:grille locale ou 2: grille adverse
\return -1 en cas d'erreur
*/
int init_grille(WINDOW* fenetre, WINDOW* grille_jeu[10][10], int x, int y, int choix){

	int colonne,ligne;
	int i,j;

	// Creer la fenetre (hauteur:26; largeur:38; y:7; x:1)
	fenetre = newwin(26,38,y,x);

	// titre et contour de la fenetre en couleur N0 (Blanc et Noir)
	wbkgd(fenetre,COLOR_PAIR(0));
	box(fenetre,0,0);
	if(choix==1){
		mvwprintw(fenetre,0,3,"| Flotte locale |");
	}else if(choix==2){
		mvwprintw(fenetre,0,3,"| Flotte Adverse |");
	}else{
		return -1;
	}

	// Contenu de la fenetre en couleur N3 (Blanc et bleu)
	wattron(fenetre,COLOR_PAIR(3));
	mvwprintw(fenetre,2,2,"   A  B  C  D  E  F  G  H  I  J ");
	mvwhline(fenetre,3,2,ACS_HLINE,32);

	// Ajouter les lignes horizontales
	for(ligne=4;ligne<24;ligne+=2){
		if(ligne/2-1 != 10){
			mvwprintw(fenetre,ligne,2," %d",ligne/2-2);
		}else{
			mvwprintw(fenetre,ligne,2," %d",ligne/2-2);
		}
		mvwhline(fenetre,ligne+1,2,ACS_HLINE,32);
	}

	// Ajouter les lignes verticales
	mvwvline(fenetre,2,4,ACS_VLINE,22);
	for(colonne=7;colonne<36;colonne+=3){
		mvwvline(fenetre,2,colonne,ACS_VLINE,22);
	}

	// Fin de la couleur N3
	wattroff(fenetre,COLOR_PAIR(3));

	// Actualiser la fenetre
	wrefresh(fenetre);
	
	ligne = y+4;

	for(i=0;i<10;i++){
		colonne = x+5;
		for(j=0;j<10;j++){
			grille_jeu[i][j] = newwin(1,2,ligne,colonne);
			wbkgd(grille_jeu[i][j],COLOR_PAIR(2));
			mvwprintw(grille_jeu[i][j],1,1," ");
	
			wrefresh(grille_jeu[i][j]);
			colonne+=3;
		}
		ligne+=2;
	}

	return 1;
}


//! Initialiser l'affichage ncurses
void init_screen(){

	WINDOW* flotte_adverse;
	WINDOW* flotte_locale;
	
	// Initialiser l'cran
	initscr();
	
	// Dfinir les couleurs
	if(can_change_color()){
		mvprintw(2,(COLS-15)/2,"Votre console ne supporte pas la couleur");
	}else{
		// Dmarrer le mode couleur
		start_color();
		// Texte gnral en bleu sur fond noir
		init_pair(1,COLOR_BLUE,COLOR_BLACK);
		// Grille de jeu
		init_pair(2,COLOR_BLACK,COLOR_BLUE);
		// Grille de jeu 2
		init_pair(3,COLOR_WHITE,COLOR_BLUE);
		// Bateau
		//init_color(COLOR_GREY,165,152,152);
		init_pair(4,COLOR_WHITE,COLOR_BLACK);
		// Bateau touche
		init_pair(5,COLOR_WHITE,COLOR_YELLOW);
		// Bateau coule
		init_pair(6,COLOR_WHITE,COLOR_RED);
	}
		
	// Pointeur vers la structure constituant la fentre
	WINDOW *fenetre_generale;
	
	// Creer la fenetre d'affichage des messages
	fenetre_generale = newwin(0,0,0,0);

	// Definir les bordues de la fenetre
	box(fenetre_generale,0,0);
	
	// Definir le fond de la fenetre
	wbkgd(fenetre_generale,COLOR_PAIR(1));

	// Titre de la fenetre
	mvwprintw(fenetre_generale,0,3,"| SeaBattle |");

	// Rafraichir l'ecran
	wrefresh(fenetre_generale);
	
	// Affiche flotte et grille locale
	refresh();

	// Fenetre pour la liste des bateaux  placer
	bateau = newwin(19,28,2,COLS/2-28/2);

	// Definir les bordues de la fenetre
	box(bateau,0,0);
	
	// Definir le fond de la fenetre
	wbkgd(bateau,COLOR_PAIR(0));

	// Titre de la fenetre
	mvwprintw(bateau,0,5,"| SeaBattle 1.0 |");
	mvwprintw(bateau,2,5,"Didier BERNAUDEAU");
	mvwprintw(bateau,3,6,"Julien VEHENT");
	mvwprintw(bateau,4,1,"2006 - aucun droit rserv");

	mvwhline(bateau,6,2,ACS_HLINE,22);

	mvwprintw(bateau,8,2,"Bateaux disponibles:");
	mvwprintw(bateau,9,3,"1 - Porte-avion");
	mvwprintw(bateau,10,3,"2 - Croiseur n1");
	mvwprintw(bateau,11,3,"3 - Croiseur n2");
	mvwprintw(bateau,12,3,"4 - Croiseur n3");
	mvwprintw(bateau,13,3,"5 - Corvette n1");
	mvwprintw(bateau,14,3,"6 - Corvette n2");
	mvwprintw(bateau,16,6,"Tapez /help pour");
	mvwprintw(bateau,17,5,"obtenir de l'aide");
	
	// Rafraichir l'ecran
	wrefresh(bateau);
	refresh();
	
	// Grille
	init_grille(flotte_locale,grille_locale,COLS/4-38/2,2,1);
	init_grille(flotte_adverse,grille_adverse,COLS/4*3-38/2,2,2);
}

//! Initialiser l'affichage ncurses
/*!
\param[in,out] tab_message est un tableau contenant les messages. Il sera uniquement initialise
\return la fenetre de chat. Utilisee pour modifier son contenu.
*/
WINDOW* init_chat(char tab_message[9][200]){

	// Compteur de ligne
	int i;
	
	// Pointeur vers la structure constituant la fentre
	WINDOW *fenetre_chat;
	
	// Initialiser le tableau des messages
	for(i=0;i<10;i++) strcpy(tab_message[i],"");

	// Creer la fenetre d'affichage des messages
	fenetre_chat = newwin(11,COLS-3,34,1);

	// Definir les bordues de la fenetre
	box(fenetre_chat,0,0);
	
	// Definir le fond de la fenetre
	wbkgd(fenetre_chat,COLOR_PAIR(0));

	// Titre de la fenetre
	mvwprintw(fenetre_chat,0,3,"| Dialogue |");
		
	// Rafraichir l'ecran
	wrefresh(fenetre_chat);

	// Positionner le curseur  l'emplacement de saisie sur l'cran principal
	move(36,3);

	return fenetre_chat;
}

//! Afficher les messages du chat
/*!
\param[in] tab_message est un tableau contenant les messages a afficher
\param[in] fenetre_chat est un pointeur vers la fenetre du chat.
*/
void afficher_chat(char tab_message[9][200], WINDOW* fenetre_chat){

	// Compteur de ligne du tableau
	int i;

	// Effacer les caracteres eventuellement afficher sur cette meme ligne
	wclear(fenetre_chat);
	
	// Refaire les bordues de la fenetre
	box(fenetre_chat,0,0);

	// Titre de la fenetre
	mvwprintw(fenetre_chat,0,3,"| Dialogue |");
	
	// Affichage des message
	for(i=0;i<9;i++) {
		mvwprintw(fenetre_chat,i+1,1,tab_message[i]);
	}

	// placer le curseur pour la saisie
	move(LINES-4,2);
	
	// Rafraichir la fenetre
	wrefresh(fenetre_chat);
}

//! Afficher un message d'erreur
/*!
\param[in] msg est la chaine de caracteres a afficher
*/
void afficher_erreur(char* msg){

	erreur = newwin(6,COLS-2,28,1);

	// Definir les bordues de la fenetre
	box(erreur,0,0);
	
	// Definir le fond de la fenetre
	wbkgd(erreur,COLOR_PAIR(0));

	// Titre de la fenetre
	mvwprintw(erreur,0,3,"| Message du jeu |");
	
	mvwprintw(erreur,2,(COLS-strlen(msg)-2)/2,"%s",msg);

	wrefresh(erreur);
}

//! Effacer les messages d'erreur
void effacer_erreur(){
	wclear(erreur);
	wrefresh(erreur);
}

//! Enregistrer les messages dans un tableau
/*!
\param[in,out] tab_message est un tableau dans lequel sera insere le message.
\param[in] fenetre_chat est un pointeur vers la fenetre du chat.
\param[in] message est une chaine de 200 caract?es contenant le message envoye par l'utilisateur
*/
void saisie(char tab_message[9][200], WINDOW* fenetre_chat, char *message){

	// compteur et indicateur de la derni?e ligne vide
	int i,ligne_vide;

	// Effacer les caracteres eventuellement afficher sur cette meme ligne
	for(i=2;i<5;i++){
		move(LINES-i,1);
		clrtoeol();
	}

	// Possitionner le curseur pour la saisie du message
	move(LINES-4,2);
	
	// Recherche la derniere ligne pleine du tableau (en partant de la fin)
	for(i=9;(strcmp(tab_message[i],"")==0) && (i>=0);i--){mvprintw(i+2, 5, "%d",i);refresh();}

	// Si le tableau est plein
	if(i==9){
		// remonter les lignes du tableau
		for(i=0;i<9;i++) strcpy(tab_message[i],tab_message[i+1]);
		// la ligne vide et la derniere ligne du tableau
		ligne_vide = 8;
	}else{
		// Si le tableau n'est pas plein, la ligne vide suit la derniere ligne pleine
		ligne_vide = i+1;
	}

	// Saisir le message dans le tableau
	// getstr(tab_message[ligne_vide]);
	strcpy(tab_message[ligne_vide],message);

	// Afficher le tableau
	afficher_chat(tab_message,fenetre_chat);
}

//! Change la couleur des cases en fonction de leur etat
/*!
\param[in] choix 1:local et 2:adverse
\param[in] matrice etat des cases
*/
void afficher_matrice(int choix, int matrice[10][10]){

	int i,j,matrice_temp;

	if(choix==1){
		/// Matrice locale
		for(i=0;i<10;i++){
			for(j=0;j<10;j++){
				matrice_temp = matrice[i][j];
				switch(matrice_temp){

					/// Case vide = bleu
					case 0 : wbkgd(grille_locale[i][j],COLOR_PAIR(2));break;
			
					/// Case vide touchee = bleu avec une croix blanche
					case 1 : wbkgd(grille_locale[i][j],COLOR_PAIR(3));
						 mvwprintw(grille_locale[i][j],0,0,"X");
						 break;

					/// Case contenant un bateau = noir
					case 4 : case 5 : case 6: case 7: case 8: case 9: 
						 wbkgd(grille_locale[i][j],COLOR_PAIR(4));
						 break;

					/// Case contenant un bateau coule = rouge
					case -14: case -15 : case -16 : case -17 : case -18 : case -19 : 
							wbkgd(grille_locale[i][j],COLOR_PAIR(6));
							break;

					/// Case contenant un bateau touche = jaune
					case -4 : case -5 : case -6 : case -7 : case -8: case -9 : 
							wbkgd(grille_locale[i][j],COLOR_PAIR(5));
							break;
							
					default : afficher_erreur("erreur de refresh de la matrice locale");break;

				}
				wrefresh(grille_locale[i][j]);refresh();
			}
		}
	}else{
		/// Matrice adverse
		if (choix ==2){
			for(i=0;i<10;i++){
				for(j=0;j<10;j++){
					matrice_temp = matrice[i][j];
					switch(matrice_temp){
	
						/// Case vide = bleu
						case 0 : wbkgd(grille_adverse[i][j],COLOR_PAIR(2));break;
				
						/// Case vide touchee = bleu avec une croix blanche
						case 1 : wbkgd(grille_adverse[i][j],COLOR_PAIR(3));
							mvwprintw(grille_adverse[i][j],0,0,"X");
							break;
	
						/// Case contenant un bateau coule = rouge
						case -14: case -15 : case -16 : case -17 : case -18 : case -19 : 
							wbkgd(grille_adverse[i][j],COLOR_PAIR(6));
							break;
	
						/// Case contenant un bateau touche = jaune
						case -4: case -5 : case -6 : case -7 : case -8 : case -9  : 
							wbkgd(grille_adverse[i][j],COLOR_PAIR(5));
							break;
					default : afficher_erreur("erreur de refresh de la matrice adverse");break;
					}
					wrefresh(grille_adverse[i][j]);refresh();
				}
			}
		}
	}
}

//! Effacer le bateau de liste des bateaux disponible
void effacer_bateau(int id_bateau){

	switch(id_bateau){

		case 1 :wmove(bateau,9,3);break;
		case 2 :wmove(bateau,10,3);break;
		case 3 :wmove(bateau,11,3);break;
		case 4 :wmove(bateau,12,3);break;
		case 5 :wmove(bateau,13,3);break;
		case 6 :wmove(bateau,14,3);break;		
	}

	wclrtoeol(bateau);

	// Definir les bordures de la fenetre
	box(bateau,0,0);
	mvwprintw(bateau,0,5,"| SeaBattle 1.0 |");
	wrefresh(bateau);
}

//! Affiche l'tat du jeu
/*!
\param[in] code est la rfrence du message  afficher
*/
void afficher_etat(int code){

	// Effacer le texte
	wmove(bateau,8,2);
	wclrtobot(bateau);

	// Cadre et titre de la fenetre
	box(bateau,0,0);
	mvwprintw(bateau,0,5,"| SeaBattle 1.0 |");

	switch(code){
		case 0 :
			mvwprintw(bateau,9,6,"C'est  votre");
			mvwprintw(bateau,10,4,"adversaire de jouer");
			break;
		case 1 :
			mvwprintw(bateau,9,6,"C'est  votre");
			mvwprintw(bateau,10,6,"tour de jouer");
			mvwprintw(bateau,15,7,"Pour tirer :");
			mvwprintw(bateau,16,2,"/shoot colonne ligne");
			mvwprintw(bateau,17,2,"ex: /shoot B 5");
			
			break;
		case 10 : case -10 :
			mvwprintw(bateau,9,7,"La partie");
			mvwprintw(bateau,10,7,"termine");
			break;
	}
	move(LINES-4,2);
	wrefresh(bateau);
}