/*******************************************************************************\
 *			SeaBattle - D.Bernaudeau, J. Vehent - 2006 	       *
 *									       *
 *	fonction reseau: tablissement des connexions, boucle dans l'envoi     *
 *		et la rception des messages, envoi et recoit les tirs	       *
 *									       *
\*******************************************************************************/ 

/*! \file reseau.c
    \brief fonctions reseau

    tablissement des connexions, boucle dans l'envoi et la rception des messages, envoi et recoit les tirs	

    \Author D.Bernaudeau
    \Author  J. Vehent
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/wait.h>
#include <signal.h>
#include <netinet/in.h>
#include <netdb.h>
#include <pthread.h>
#include <ncurses.h>

#include "reseau.h"
#include "affichage.h"
#include "jeu.h"

/*!
  \def PORTECOUTE
  \brief Port en coute pour le serveur
 */
#define PORTECOUTE	1664

/*!
  \def MAXDATASIZE
  \brief Quantit max d'octets que l'on peut envoyer/recevoir
 */
#define MAXDATASIZE	200

/*!
  \def MAXCONNECT
  \brief Nombre max de connection que le serveur accepte d'un coup
 */
#define MAXCONNECT	1

/*******VARIABLES GLOBALES*******/

/*!
  \var fenetre_chat
  \brief fenetre ncurse pour l'affichage du chat
 */
WINDOW* fenetre_chat;

/*!
   \var tab_message
   \brief Tableau contenant les messages du chat
 */
char	tab_message[9][200];

/*!
   \var quitter
   \brief Variable mise a vrai pour quiter le jeu
 */
bool quitter=false;

/*!
   \var socket_fd
   \brief Identifiant du socket rseau
 */
int socket_fd;

/*********************************/


//! Configuration du socket rseau, utilise le socket en variable globale
void config_reseau(){

	int yes=1;
	
	/* Creation du socket file descriptor local
	 * sa socket_fd = socket(int domain, int type, int protocol)
	 * param socket_fd fichier de socket
	 * param domain entier, specifie le type d'adresse (PF_INET == AF_INET)
	 * param type entier, specifie le type de socket (STREAM == connection (tcp))
	 * param protocol entier, specifie le flag de protocol (0 == mode auto)
	 *
	 * return -1 en cas d'erreur (test avec un if), sinon elle renvoie un socket file descriptor
	 */
	if ((socket_fd = socket(PF_INET, SOCK_STREAM, 0)) == -1) {
	      		perror("socket");
	      		exit(EXIT_FAILURE);
	}
	
	/* Modification des parametres d'un socket 
	 * sa setsockopt(int s, int level, int optname, const void *optval, socklen_t optlen)
	 * param s socket file descriptor
	 * param level au niveau socket, ce paramtre prend la valeur SOL_SOCKET
	 * param optname option que l'on modifie, ici c'est SO_REUSEADDR pour la reutilisation du socket
	 * param *optval pointeur vers la nouvelle valeur, ici 1
	 * param optlen option de longueur, ici la taille d'un int
	 * return -1 en cas d'erreur (test dans un if), sinon rien
	 */
	if (setsockopt(socket_fd,SOL_SOCKET,SO_REUSEADDR,&yes,sizeof(int)) == -1) {
		perror("setsockopt");
	        exit(EXIT_FAILURE);
	}
	
	
}

/***************************************************************************************************************/

//! Envoi un tir a l'adversaire via le rseau
/*!
 \param[in] objective chaine de caractre contenant les coordonnes de la case adverse vise
 */
void send_shoot(char *objective){
	int lenobj;
	lenobj = strlen(objective);
	if (send(socket_fd, objective, lenobj, 0) == -1) perror("send");
}


/***************************************************************************************************************/

//! Renvoi la rponse  un tir reu
/*!
 \param[in] result un entier correspondant a l'tat de la case vise
*/
void send_rep_shoot(int result){
	
	char rep[5]="";
	
	//on convertit l'entier en char pour l'envoyer sur le rseau
	sprintf(rep,"%d",result);
	
	int replen = strlen(rep);
	if (send(socket_fd, rep, replen, 0) == -1) perror("send");
}


/***************************************************************************************************************/

//! Reception des messages sur le socket, la fonction tourne en boucle et coute le socket puis traite le message recu
void recv_msg(){
	int octets_recus;
	char buf[MAXDATASIZE]=".", quit[]="/quit", go[]="/go", shoot[]="/shoot", slash[]="/";

	do{
		
		//on place le socket en mode recv pour recevoir les messages
		if ((octets_recus=recv(socket_fd, buf, MAXDATASIZE-1,0)) == -1){
			perror("recv");
			exit(EXIT_FAILURE);
		}
		buf[octets_recus]='\0';
		
		/* ZONE DE CONTROLES DES CHAINES DE CARACTERES
		 *
		 * avant d'afficher le message reu comme un texte de conversation, 
		 * on vrifie que ce n'est pas un message particulier (dbutant par un slash).
		 *
		 * Si c'est le cas, alors on test le cas du /go, puis celui du /shoot
		 * enfin celui du /quit
		 *
		 * si le message n'est aucun de ceux la, alors on test si c'est la rponse a
		 * un tir, sinon on affiche comme du texte
		 */
		if (0 == strncmp(buf, slash,1)){
		
			//test si la notification /go est recue
			//si c'est le cas alors l'adversaire et pret : on appel adv_go()
			if (0 == strcmp(buf,go)) adv_go();
	
			else{ 	//test de la reception d'un tir ennemi
				//on l'envoi directement a la fonction qui le traite
				if(0 == strncmp(buf,shoot,6)) recv_shoot(buf);

				/* On test si on recoit un /quit (fermeture de connection)
		 		* si c'est le cas, on place le booleen "quitter" a true pour
	 			* sortir du while
	 			*/
				else if(0 == strncmp(buf, quit, 5))quitter=true;
			}
			//test si le message est une rponse a un tir
		}else 	if (strlen(buf) <= sizeof(int)){
				rep_shoot(buf);
			}
			//envoi du message dans la fonction qui gre le tableau et l'affichage
			else saisie(tab_message, fenetre_chat, buf);
		
	}while( !quitter );

	//quand on quitte la discussion, on affiche un message de fermeture avec un sleep
	//(histoire d'avoir le temps de le lire :p)
	afficher_erreur("FERMETURE DE LA CONNECTION DEMANDEE, passez une bonne journee ;)");
	sleep(2);
}


/***************************************************************************************************************/

//! Envoi des messages sur le socket. Fonction appele dans un thread, elle tourne en boucle et rcupre la saisie clavier pour l'analyser et l'envoyer si besoin
void send_msg(){
	
	char msg[MAXDATASIZE]="", phrase[MAXDATASIZE+11], go[]="/go", quit[]="/quit", slash[]="/";
	int len;
	//tableau de char pour le nom
	char nom[16]; int lennom;

	//saisie du pseudo de l'utilisateur au dbut de la partie
	mvprintw(LINES-5, 1, "Tapez votre pseudo : ");
	move(LINES-4,2);
	refresh();
	getstr(nom);
	lennom = strlen(nom);
	nom[lennom]='\0';
	strcat(nom, "> ");
	lennom = strlen(nom);
	nom[lennom]='\0';

	//Initialisation de la saisie d'un message
	move(LINES-5,1);clrtoeol();
	mvprintw(LINES-5,1,"Votre message :");
	move(LINES-4,2);clrtoeol();
	refresh();
	
	/* BOUCLE INFINIE TANT QUE L'UTILISATEUR N'A PAS SAISI /QUIT
	 */
	while(strcmp(msg, quit)){
		
		//saisie du message a envoyer
		getstr(msg);
		
		//suppression de la fenetre des messages d'erreur
		effacer_erreur();

		/* CONTROLE DES CHAINES DE CARACTERES
		 *
		 * on test si la saisie commence par un slash, si c'est le cas on passe en analyse
		 *
		 * sinon on envoie le message comme du texte
		 */


		//le premier caractere est un slash, on analyse la saisie
		if (0 == strncmp(msg,slash,1)){
	
			//la chaine saisie est /go
			if(0 == strcmp(msg, go)){
				len=strlen(msg);
				
				//test du message dans la fonction d'interpretation
				//elle renvoie -2 si le joueur n'a pas plac tous ses bateaux
				//et donc ne peut pas dmarrer
				if (!(-2 == interpretation(msg))){
					if (send(socket_fd, msg, len, 0) == -1) perror("send");}
			}
			
			/* Demande de fermeture de connection
			 * le message est envoy tel quel dans la socket
			 */
			if(0 == strcmp(msg, quit)){
				len=strlen(msg);
				if (send(socket_fd, msg, len, 0) == -1) perror("send");
			}
			else{
				if( -1 == interpretation(msg) ){
					afficher_erreur("Commande inconnue ou mal formule");
				}
			}

			//affichage de la saisie dans ncurses
			saisie(tab_message, fenetre_chat, msg);

		}
		else {
			/* Si l'on est dans ce cas, c'est que le message saisie est une discussion
			 * normale, on l'envoi au travers de la socket
			 */
			
			//dans le tableau "phrase" on met d'abord le nom de l'emetteur
			strcpy(phrase, nom);
			
			//on met ensuite le texte a envoyer
			strcat(phrase, msg);
		
			//et on calcul de la longueur de la phrase
			len=strlen(phrase);
			
			/* Envoi du message au travers de la socket rseau
			 * sa send(int s, const void *buf, size_t len, int flags)
			 * param s socket file descriptor
			 * param *buf pointeur vers le tableau de caractres a envoyer
			 * param len longueur de la chaine a envoyer
			 * param flags drapeau non utilis
			 * return -1 en cas d'erreur (test dans un if), rien sinon
			 */
			if (send(socket_fd, phrase, len, 0) == -1) afficher_erreur("erreur lors de l'envoi du message"); 
				
			//affichage dans ncurses
			//en local une fois que le send a reussi
			saisie(tab_message, fenetre_chat, phrase);
		}
	
	}
	//quand on sors, on place le booleen quitter a true pour fermer le socket recv
	quitter=true;
	//sortie du thread une fois la commande /quit saisie
	pthread_exit(0);
}	


/***************************************************************************************************************/

//! Client : Configure le rseau et appelle send_msg (dans un thread) et recv_msg dans le programme principal
/*!
\param[in] IP l'adresse IP du serveur a contacter
*/
int client(int IP){

	int i;

	/* les structures de parametrages reseau */
	struct sockaddr_in my_addr_clt;
	struct sockaddr_in their_addr_srv;
	/* structure de creation du thread */
	pthread_t thread_send_msg;
	
	/* Initialiser le tabeau des messages
	 */
	for(i=0;i<10;i++){
		strcpy(tab_message[i],"");
	}
	
	/* appel de la fonction de configuration reseau
	 * on passe l'adresse du socket en parametre
	 *
	 * le socket est modifie directement dans la memoire
	 */
	config_reseau();
	
	
	/* my_addr_clt()
	 * param sin_family domaine d'adresse, octet dans l'ordre de l'hote
	 * param sin_addr.s_addr adresse source recuperee de l'OS
	 * param memset mise a \0 de la fin de la structure
	 *
	 */
	my_addr_clt.sin_family = AF_INET;
	my_addr_clt.sin_addr.s_addr = INADDR_ANY;
	memset(&(my_addr_clt.sin_zero), '\0', 8);
	

	/* their_addr_srv()
	 * configuration reseau du serveur auquel on se connecte
	 * le port de destination est converti en ecriture reseau par la fonction
	 * htons() => host to network short
	 */
	their_addr_srv.sin_family = AF_INET;
	their_addr_srv.sin_port = htons(PORTECOUTE);
	their_addr_srv.sin_addr.s_addr = inet_addr( (void *)IP);
	memset(&(their_addr_srv.sin_zero), '\0', 8);


	/* connect() la fonction connect renvoie -1 en cas d'echec
	 * param clt_recv_fd socket file descriptor
	 * param their_addr_srv pointeur vers la structure struct sockaddr
	 * param addrlen taille de la structure sockaddr
	 */
	if (connect(socket_fd, (struct sockaddr *)&their_addr_srv,
				sizeof(struct sockaddr)) == -1){
		perror("connect");
		exit(EXIT_FAILURE);
	}

	init_screen();
	
	//Initialisation de la fenetre de discussion
	fenetre_chat = init_chat(tab_message);
	
	//initialisation des matrices de jeu
	init_matrices();

	/* pthread_create() processus parallele pour l'envoi de messages
	 * param &thread_send_msg structure contenant le thread
	 * param NULL argument du thread laissee par defaut
	 * param send_msg fonction executee par le thread (transtypee en void *)
	 * param socket_fd socket file descriptor passee en argument (transtypee en int *)
	 */
	if( 0!= pthread_create (&thread_send_msg, 
					NULL, 
					(void *)send_msg,
					NULL) ){
		perror("creation du thread");
		exit(EXIT_FAILURE);

	}

	/* pthread_detach() vidage du thread de la memoire des qu'il termine
	 * param thread_send_msg structure contenant le thread
	 * cette fonction permet de detacher le thread du programme car on ne 
	 * cherche pas a recuperer de valeur a la fin de son execution
	 * il sera donc nettoye automatiquement
	 */
	pthread_detach(thread_send_msg);
	
	/* le programme continue dans le processus pere	
	 * appel de la fonction de reception des messages
	 */
	recv_msg();

	//DESTRUCTION DE L'AFFICHAGE NCURSE
	delwin(fenetre_chat);
        endwin();

        return 0;

}


/***************************************************************************************************************/

//! Serveur : Configure le rseau et appelle send_msg (dans un thread) et recv_msg dans le programme principal
int serveur(){
	
	/* les structures de parametrages reseau */
	struct sockaddr_in my_addr_srv;
	struct sockaddr_in their_addr_clt;
	/* structure de creation du thread */
	pthread_t thread_send_msg;
	/*les variables diverses */
	int sin_size;

	config_reseau();
	
	/* my_addr_srv()
	 * param sin_family domaine d'adresse, octet dans l'ordre de l'hote
	 * param sin_addr.s_addr adresse source recuperee de l'OS
	 * param sin_port port source
	 * param memset mise a 0 de la fin de la structure
	 */
	my_addr_srv.sin_family = AF_INET;
	my_addr_srv.sin_addr.s_addr = INADDR_ANY;
	my_addr_srv.sin_port = htons(PORTECOUTE);
	memset(&(my_addr_srv.sin_zero), '\0', 8);

	
	// On attend la connection du client sur le port PORTECOUTE
	if (bind(socket_fd, (struct sockaddr *)&my_addr_srv, 
				sizeof(struct sockaddr))==-1){
		perror("bind");
		exit(1);
	}
	
	fprintf(stdout,"\n\nEn attente de connection................");
	
	if (listen(socket_fd, MAXCONNECT) == -1) {
		perror("listen");
		exit(1);
	}
	
	/* accept()
	 * param socket_fd socket file descriptor retournee par la fonction socket()
	 * param their_addr_clt pointeur vers la structure sockaddr du client
	 * param sin_size taille de la structure sockaddr
	 * return -1 en cas d'erreur (test dans un if), rien sinon
	 */
	sin_size = sizeof(struct sockaddr_in);
	
	if((socket_fd = accept(socket_fd, (struct sockaddr *)&their_addr_clt,&sin_size))==-1){
		perror("accept");
		exit(1);
	}
	
	// UNE FOIS LA CONNECTION ETABLIT, ON DEMARRE L'AFFICHAGE NCURSE
	init_screen();
	
	fenetre_chat = init_chat(tab_message);
		
	//initialisation des matrices de jeu
	init_matrices();

	/* pthread_create() processus parallele pour l'envoi de messages
	 * param &thread_send_msg structure contenant le thread
	 * param NULL argument du thread laissee par defaut
	 * param send_msg fonction executee par le thread (transtypee en void *)
	 * param socket_fd socket file descriptor passee en argument (transtypee en int *)
	 */
	if( 0!= pthread_create (&thread_send_msg, 
					NULL, 
					(void *)send_msg,
					NULL) ){
		perror("creation du thread");
		exit(EXIT_FAILURE);

	}

	/* pthread_detach() vidage du thread de la memoire des qu'il termine
	 * param thread_send_msg structure contenant le thread
	 * cette fonction permet de detacher le thread du programme car on ne 
	 * cherche pas a recuperer de valeur a la fin de son execution
	 * il sera donc nettoye automatiquement
	 */
	pthread_detach(thread_send_msg);
	
	//dans le processus pere on recoit les messages et on les affiches
	recv_msg();
	
	//DESTRUCTION DE L'AFFICHAGE NCURSE
	delwin(fenetre_chat);
        endwin();

	return 0;
}

